// Copyright (C) 2018 Michel Sabourin 
//
// Sous licence du site SimTurb à : 
// https://michelsabourin.scenari-community.org/SimTurb/co/1siteWeb_1.html
//
// Date : 2018-08-28
//Version 3
//Date : 2019-11-12
// Écriture colline quadrillée
//Date : 2020-04-26
// Version pour Scilab 6.1.0
function [FileName, PathName]=LectureColline()
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
    [FileName, PathName, FilterIndex]=uigetfile("*.xls","%USERPROFILE%","Fichier de la colline?");
if FilterIndex==0 then;
        abort;
end;
Fichier=fullfile(PathName,FileName);
Feuilles=readxls(Fichier);
Colline=Feuilles(1).value;
endfunction

function LectureSession()
        global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
    [fd_r,err]=mopen(Session,"rt");

    if err==0 then // on prend les valeurs précédentes si elles exitent
    //    Lignes=string(1:18);
        [n,Option,dX,dY,dZ1,dZ2,cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max]=mfscanf(fd_r,"%d  %f %f %f %f %f %f %f %f %f %f %f %f \n");
        select Option
        case 0 // Colline d'analyse
            cX=1;//ned
            cY=3;//Qed
            cZ1=4;//eta
            cZ2=2;//gamma
        case 1 // Colline de production
            cX=1;//ned
            cY=2;//gamma
            cZ1=4;//eta
            cZ2=3;//Qed
        end
        
        Limites=formate_Limites(cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2);
    
        [n,Vue]=mfscanf(fd_r,"%d \n");
    else
        Option=0;
        cX=1;//ned
        cY=3;//Qed
        cZ1=4;//eta
        cZ2=2;//gamma
        Vue=1;
        [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max]=extremum(Colline,cX,cY,cZ1,cZ2);
        dX=(cXmax-cXmin)/10;
        dY=(cYmax-cYmin)/10;
        dZ1=(cZ1max-cZ1min)/10;
        dZ2=(cZ2max-cZ2min)/10;
        Limites=formate_Limites(cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2);
    end
endfunction

function [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max]=extremum(Colline,cX,cY,cZ1,cZ2)
    //calcul des extremum de la colline
    cXmin=min(Colline(:,cX));//minimum colonne X
    cXmax=max(Colline(:,cX));//maximum colonne X
    cYmin=min(Colline(:,cY));
    cYmax=max(Colline(:,cY));
    cZ1min=min(Colline(:,cZ1));
    cZ1max=max(Colline(:,cZ1));
    cZ2min=min(Colline(:,cZ2));
    cZ2max=max(Colline(:,cZ2));
endfunction

function Limites=formate_Limites(cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2)
    //mise en place d"une matrice String
    ligne1=[" " "Min" "Max" "Pas"];
    if Option==0 then   
        colonne1=["ned" "Qed" "Eta" "Ouv"]';
    else
        colonne1=["ned" "Ouv" "Eta" "Qed"]';
    end
    colonne2=string([cXmin cYmin cZ1min cZ2min]');
    colonne3=string([cXmax cYmax cZ1max cZ2max]');
    colonne4=string([dX dY dZ1 dZ2]');
    Limites=[ligne1;[colonne1 colonne2 colonne3 colonne4]];
endfunction

function [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2]=deformate_Limites(Limites)
    //recupération de la matrice string et mise en place les limites du domaine et des pas d"interpolation
    cXmin=evstr(Limites(2,2));
    cXmax=evstr(Limites(2,3));
    dX=evstr(Limites(2,4));
    cYmin=evstr(Limites(3,2));
    cYmax=evstr(Limites(3,3));
    dY=evstr(Limites(3,4));
    cZ1min=evstr(Limites(4,2));
    cZ1max=evstr(Limites(4,3));
    dZ1=evstr(Limites(4,4));
    cZ2min=evstr(Limites(5,2));
    cZ2max=evstr(Limites(5,3));
    dZ2=evstr(Limites(5,4));
endfunction

function calcul_Zcoef
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//On calcule les paramètres de normalisation de la colline
[etamax,ligneDuMax]=max(Colline(:,4));

//On prépare l"interpolation en extrayant les coefficients
//
//    Z1coef=cshep2d([Colline(:, cX) Colline(:,cY) Colline(:,cZ1)]);
//    Z2coef=cshep2d([Colline(:, cX) Colline(:,cY) Colline(:,cZ2)]);
//Pour le calcul sur la colline normalisée, les données sont divisées par leur valeur à la ligne du maximum de rendement
    Z1coef=cshep2d([Colline(:, cX)/Colline(ligneDuMax, cX) Colline(:,cY)/Colline(ligneDuMax, cY) Colline(:,cZ1)/Colline(ligneDuMax, cZ1)]);
    Z2coef=cshep2d([Colline(:, cX)/Colline(ligneDuMax, cX) Colline(:,cY)/Colline(ligneDuMax, cY) Colline(:,cZ2)/Colline(ligneDuMax, cZ2)]);

endfunction

function calcul_XYZ
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//Calcul de la grille
    [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2]=deformate_Limites(Limites);

    nX=1+(cXmax-cXmin)/dX;
    if nX>floor(nX) then
        nX=floor(nX)+1;
        cXmax=cXmin + dX*(nX-1);
    end
    X=[];
    X=[cXmin:dX:cXmax];

    nY=1+(cYmax-cYmin)/dY;
    if nY>floor(nY) then
        nY=floor(nY)+1;
        cYmax=cYmin + dY*(nY-1);
    end
    Y=[];
    Y=[cYmin:dY:cYmax];

    Z1=[cZ1min:dZ1:cZ1max];
    Z2=[cZ2min:dZ2:cZ2max];

    nZ1=size(Z1,"*");
    nZ2=size(Z2,"*");
    XYZ1=[];
    XYZ2=[];
// Paramètre de la normalisation de la colline
    [etamax,ligneDuMax]=max(Colline(:,4));
    for i=1:nX;
        for j=1:nY;
//Pour le calcul sur la colline normalisée, les données sont divisées par leur valeur à la ligne du maximum de rendement
            XYZ1(i,j)=Colline(ligneDuMax, cZ1)*eval_cshep2d(X(i)/Colline(ligneDuMax, cX), Y(j)/Colline(ligneDuMax, cY), Z1coef);
            XYZ2(i,j)=Colline(ligneDuMax, cZ2)*eval_cshep2d(X(i)/Colline(ligneDuMax, cX), Y(j)/Colline(ligneDuMax, cY), Z2coef);
        end
    end
//
//  Calcul de l"interpolation des séries de points s"il y a lieu
//
//    calcul_XYZ_Series()
endfunction

function calcul_XYZ_Series()
        global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//
//  Calcul de l"interpolation des séries de points s"il y a lieu
//
    nSeries=size(Series,"*");
    [etamax,ligneDuMax]=max(Colline(:,4));
    for i=1:nSeries
        serie=[];
        serie=Series(i).Value;
        [n,m]=size(serie);
        for j=1:n
//            serie(j,3)=eval_cshep2d(serie(j,1),serie(j,2),Z1coef);
//            serie(j,4)=eval_cshep2d(serie(j,1),serie(j,2),Z2coef);
            serie(j,3)=Colline(ligneDuMax, cZ1)*eval_cshep2d(serie(j,1)/Colline(ligneDuMax, cX),serie(j,2)/Colline(ligneDuMax, cY),Z1coef);
            serie(j,4)=Colline(ligneDuMax, cZ2)*eval_cshep2d(serie(j,1)/Colline(ligneDuMax, cX),serie(j,2)/Colline(ligneDuMax, cY),Z2coef);
        end
        Series(i).Value=serie;
    end

endfunction

function tracage
global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//Mise en place du graphique en 3D
//    clf();//on efface la figure
    delete(f2x.children);
//on récupère le domaine d"interpolation
    Limites=handles.Limites.String;
    [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2]=deformate_Limites(Limites);
    rect1=[cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max];
    f2x.data_bounds=[cXmin, cYmin, cZ1min; cXmax, cYmax, cZ1max ];
    //on sélectionne les points définissant la colline dans les Limites imposées
    ColtrimX=[];ColtrimY=[],ColtrimZ1=[];
    n=size(Colline,1);
    j=0;
    for i=1:n
        if  Colline(i,cX)>=cXmin & Colline(i,cX)<=cXmax && ..
            Colline(i,cY)>=cYmin & Colline(i,cY)<=cYmax && ..
            Colline(i,cZ1)>=cZ1min & Colline(i,cZ1)<=cZ1max then
            j=j+1
            ColtrimX(j)=Colline(i,cX);
            ColtrimY(j)=Colline(i,cY);
            ColtrimZ1(j)=Colline(i,cZ1);
        end
    end
//On trace les points de la colline qui se trouvent dans les limites
    scatter3d(ColtrimX,ColtrimY,ColtrimZ1);//points sur la colline
    f2x.children(1).Tag="points";
//On trace la surface
    plot3d(X,Y,XYZ1,theta,alpha,"X@Y@Z",[0,8,4],rect1);//surface de rendement
    f2x.children(1).Tag="grille";
//On trace les iso-rendements
    enfants =size(f2x.children,"*");//nombre d"entités attaché à l"axe
//Décimales de l'étiquette
    ndecimale_dZ1=ceil(-log10(dZ1));
    if ndecimale_dZ1<0 then
        ndecimale_dZ1=0;
    end
    format_dZ1="%."+string(ndecimale_dZ1)+"f";
    xset("fpf",format_dZ1);
    contour(X,Y,XYZ1,Z1,theta=theta,..          //contours rendement sur la surface des rendements
                        alpha=alpha,..
                        leg="X@Y@Z",..
                        flag=[0,1,4],..
                        ebox=rect1,..
                        zlev=cZ1max);
//Lorsque ce sera corrigé, il faudra effacer la ligne xset et ajouter l'argument string=format_dZ1 dans contour

    npolyZ1=size(f2x.children,"*")-enfants;//nombre de courbe de rendement
    polyZ1=f2x.children(1:npolyZ1);//Les iso-rendements sont stockés dans le vecteur
//On trace les iso-ouvertures
    //Décimales de l'étiquette
    ndecimale_dZ2=ceil(-log10(dZ2));
    if ndecimale_dZ2<0 then
        ndecimale_dZ2=0;
    end
    format_dZ2="%."+string(ndecimale_dZ2)+"f";
    xset("fpf",format_dZ2);
    contour(X,Y,XYZ2,Z2,theta,alpha,"X@Y@Z",[1,1,4],rect1,cZ1max);//contours ouverture sur le dessus de la boîte
    npolyZ2=size(f2x.children,"*")-npolyZ1-enfants;//nombre de courbe X2
    polyZ2=f2x.children(1:npolyZ2);//Les iso-rendements sont stockés dans le vecteur
//
//
//On trace les séries de point s"il y a lieu
//
//    tracageSeries()
endfunction

function tracageSeries()
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//on récupère le domaine d"interpolation
//    Limites=handles.Limites.String;
    [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2]=deformate_Limites(Limites);
//
//On trace les séries de point s'il y a lieu
//
    nSeries=size(Series,"*");

    for i=1:nSeries
        serie=[];
        serie=Series(i).Value;
    //on sélectionne les points dans les Limites imposées
    ColtrimX=[];ColtrimY=[];ColtrimZ1=[];
        [n,m]=size(serie);
        k=0
        if Option==OptSeries then
            
            for j=1:n
                if  serie(j,1)>=cXmin & serie(j,1)<=cXmax & ..
                serie(j,2)>=cYmin & serie(j,2)<=cYmax & ..
                serie(j,3)>=cZ1min & serie(j,3)<=cZ1max then
                k=k+1
                ColtrimX(k)=serie(j,1);
                ColtrimY(k)=serie(j,2);
                ColtrimZ1(k)=serie(j,3);
                end
            end
        else
            for j=1:n
                if  serie(j,1)>=cXmin & serie(j,1)<=cXmax & ..
                serie(j,4)>=cYmin & serie(j,4)<=cYmax & ..
                serie(j,3)>=cZ1min & serie(j,3)<=cZ1max then
                k=k+1
                ColtrimX(k)=serie(j,1);
                ColtrimY(k)=serie(j,4);
                ColtrimZ1(k)=serie(j,3);
                end
            end
            
            
        end
        
        if k>0 then
            scatter3d(ColtrimX,ColtrimY,ColtrimZ1);//points sur la colline
            hserie=f2x.children(1);
            hserie.Tag="serie"+string(i);
            hserie.line_mode = "on";
            hserie.foreground=23;
            hserie.thickness=1;
            hserie.mark_mode="off";
        end
        
    end
endfunction

function habillage
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
    boutonEcritures_callback;
    if Option==0 then
        handles.GrOuv.String="iso-Ouv";
    else
        handles.GrOuv.String="iso-Qed";
    end

//selon le type de vue:
    select Vue
    case 1
        VuePlan_callback;
        handles.VuePlan.Value=1;
    case 2
        VueEle_callback;
        handles.VueEle.Value=1;
    case 3
        VueProf_callback;
        handles.VueProf.Value=1;
    case 4
        VueIso_callback;
        handles.VueIso.Value=1;
    end
    habillageSeries()
endfunction

function habillageSeries()
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries")
//
//On hamille les séries de point s'il y a lieu
//
    nSeries=size(Series,"*");
    if nSeries>=1
        serie=findobj("Tag","serie1");
        if handles.GrSerie1VisiOui.Value==1 then
            serie.visible = "on";
        else
            serie.visible = "off";
        end
        serie.foreground=handles.GrSerie1Couleur.Value;
        serie.thickness=handles.GrSerie1Trait.Value;
    end
    if nSeries>=2
        serie=findobj("Tag","serie2");
        if handles.GrSerie2VisiOui.Value==1 then
            serie.visible = "on";
        else
            serie.visible = "off";
        end
        serie.foreground=handles.GrSerie2Couleur.Value;
        serie.thickness=handles.GrSerie2Trait.Value;
    end
    if nSeries>=3
        serie=findobj("Tag","serie3");
        if handles.GrSerie3VisiOui.Value==1 then
            serie.visible = "on";
        else
            serie.visible = "off";
        end
        serie.foreground=handles.GrSerie3Couleur.Value;
        serie.thickness=handles.GrSerie3Trait.Value;
    end
    if nSeries>=4
        serie=findobj("Tag","Serie4");
        if handles.GrSerie4VisiOui.Value==1 then
            serie.visible = "on";
        else
            serie.visible = "off";
        end
        serie.foreground=handles.GrSerie4Couleur.Value;
        serie.thickness=handles.GrSerie4Trait.Value;
    end
    if nSeries>=5
        serie=findobj("Tag","serie5");
        if handles.GrSerie5VisiOui.Value==1 then
            serie.visible = "on";
        else
            serie.visible = "off";
        end
        serie.foreground=handles.GrSerie5Couleur.Value;
        serie.thickness=handles.GrSerie5Trait.Value;
    end
endfunction

function sauveSession
    global ("Fichier","Session","Option","cX","cY","cZ1","cZ2","Z1coef","Z2coef","Limites","X","Y","Z1","Z2","XYZ1","XYZ2","theta","alpha","polyZ1","polyZ2","E","Vue","f","handles", "f2","f2x","Colline", "nomDesSeries", "Series", "OptSeries");
    mclose("all");
    fd_w=mopen(Session,"wt");
    [cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max,dX,dY,dZ1,dZ2]=deformate_Limites(Limites);
    E=get(handles.Ecritures,"String");
    mfprintf(fd_w,"%d %f %f %f %f \n",Option,dX,dY,dZ1,dZ2);
    mfprintf(fd_w,"%f %f %f %f %f %f %f %f \n",cXmin,cXmax,cYmin,cYmax,cZ1min,cZ1max,cZ2min,cZ2max);
    if  handles.VuePlan.Value==1 then
       Vue=1;
    end
    if handles.VueEle.Value==1 then
        Vue=2;
    end
    if handles.VueProf.Value==1 then
        Vue=3;
    end
    if handles.VueIso.Value==1 then
        Vue=4;
    end
    mfprintf(fd_w,"%d \n",Vue);
    mclose(Session);
//
// On prépare l'écriture de la colline quadrillée en option 0 (colline de production)
// 2 fonctions représentées , une en débit et l'autre en rendement
// Chacune des fonction dans un fichier .csv
//




    nom=strsubst(Fichier,".xls",""); //même nom que la colline lue en xls mais sera écrite en csv
    CollineDebit=nom+"Debit.csv";
    CollineRendement=nom+"Rendement.csv";
    CollineOuverture=nom+"Ouverture.csv";
// On bâtit le tableau pour être compatible avec la fonction interpo2 du tableur
//  Q= f(ouv - ned)
    ligne1=[0,X];
    colonne1=Y';
    if Option ==1 then
        fdD_w=mopen(CollineDebit,"wt");
        fdR_w=mopen(CollineRendement,"wt");
        Q=[ligne1;[colonne1,XYZ2']];
//  Eta=g(ouv - ned)
        Eta=[ligne1;[colonne1,XYZ1']];
//  Écriture et fermeture des fichiers
        csvWrite(Q,CollineDebit,";",",");
        mclose(CollineDebit);
        csvWrite(Eta,CollineRendement,";",",");
        mclose(CollineRendement);
    else
        fdR_w=mopen(CollineRendement,"wt");
        fdO_w=mopen(CollineOuverture,"wt");
        Eta=[ligne1;[colonne1,XYZ1']];
//  Eta=g(ouv - ned)
        Ouv=[ligne1;[colonne1,XYZ2']];
//  Écriture et fermeture des fichiers
        csvWrite(Eta,CollineRendement,";",",");
        mclose(CollineRendement);
        csvWrite(Ouv,CollineOuverture,";",",");
        mclose(CollineOuverture);
    end
    
//
//On prépare l'écriture des séries interpolées dans des fichier .csv
//
    nSeries=size(Series,"*");
    for i=1:nSeries
        FichierSeries=handles.FichierSeries.String;
        nom=strsubst(FichierSeries,".xls","");
        FichierCsv=nom+nomDesSeries(i)+".csv";
        fd_w=mopen(FichierCsv,"wt");
        serie=[];
//On formate l'information dans l'ordre du fichier colline
        if Option==0 then
            serie(:,1)=Series(i).Value(:,1);
            serie(:,2)=Series(i).Value(:,4);
            serie(:,3)=Series(i).Value(:,2);
            serie(:,4)=Series(i).Value(:,3);
        else
            serie(:,1)=Series(i).Value(:,1);
            serie(:,2)=Series(i).Value(:,2);
            serie(:,3)=Series(i).Value(:,4);
            serie(:,4)=Series(i).Value(:,3);
        end
        csvWrite(serie,FichierCsv,";",",");
        mclose(FichierCsv);
    end
    
endfunction
